
#ifndef DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE_H
#define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE_H

#ifdef IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_STATIC_DEFINE
#  define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE
#  define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_HIDDEN
#else
#  ifndef DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE
#    ifdef ignition_gazebo6_linearbatteryplugin_system_EXPORTS
        /* We are building this library */
#      define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE __declspec(dllexport)
#    else
        /* We are using this library */
#      define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE __declspec(dllimport)
#    endif
#  endif

#  ifndef DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_HIDDEN
#    define DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_HIDDEN 
#  endif
#endif

#ifndef IGN_DEPRECATED_ALL_VERSIONS
#  define IGN_DEPRECATED_ALL_VERSIONS __declspec(deprecated)
#endif

#ifndef IGN_DEPRECATED_ALL_VERSIONS_EXPORT
#  define IGN_DEPRECATED_ALL_VERSIONS_EXPORT DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE IGN_DEPRECATED_ALL_VERSIONS
#endif

#ifndef IGN_DEPRECATED_ALL_VERSIONS_NO_EXPORT
#  define IGN_DEPRECATED_ALL_VERSIONS_NO_EXPORT DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_HIDDEN IGN_DEPRECATED_ALL_VERSIONS
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_NO_DEPRECATED
#    define IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_NO_DEPRECATED
#  endif
#endif

#endif /* DETAIL_IGNITION_GAZEBO_LINEARBATTERYPLUGIN_SYSTEM_VISIBLE_H */
