/*
 * Copyright (C) 2017 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

//                ****** Do not modify this file. ******
// This file is automatically generated by CMake. Changes should instead be
// made to cmake/ign_auto_headers.hh.in in ignition-cmake

#include <ignition/gazebo/config.hh>
#include <ignition/gazebo/components/Actor.hh>
#include <ignition/gazebo/components/Actuators.hh>
#include <ignition/gazebo/components/AirPressureSensor.hh>
#include <ignition/gazebo/components/Altimeter.hh>
#include <ignition/gazebo/components/AngularAcceleration.hh>
#include <ignition/gazebo/components/AngularVelocity.hh>
#include <ignition/gazebo/components/AngularVelocityCmd.hh>
#include <ignition/gazebo/components/Atmosphere.hh>
#include <ignition/gazebo/components/AxisAlignedBox.hh>
#include <ignition/gazebo/components/BatterySoC.hh>
#include <ignition/gazebo/components/Camera.hh>
#include <ignition/gazebo/components/CanonicalLink.hh>
#include <ignition/gazebo/components/CastShadows.hh>
#include <ignition/gazebo/components/CenterOfVolume.hh>
#include <ignition/gazebo/components/ChildLinkName.hh>
#include <ignition/gazebo/components/Collision.hh>
#include <ignition/gazebo/components/Component.hh>
#include <ignition/gazebo/components/ContactSensor.hh>
#include <ignition/gazebo/components/ContactSensorData.hh>
#include <ignition/gazebo/components/CustomSensor.hh>
#include <ignition/gazebo/components/DepthCamera.hh>
#include <ignition/gazebo/components/DetachableJoint.hh>
#include <ignition/gazebo/components/ExternalWorldWrenchCmd.hh>
#include <ignition/gazebo/components/Factory.hh>
#include <ignition/gazebo/components/ForceTorque.hh>
#include <ignition/gazebo/components/Geometry.hh>
#include <ignition/gazebo/components/GpuLidar.hh>
#include <ignition/gazebo/components/Gravity.hh>
#include <ignition/gazebo/components/HaltMotion.hh>
#include <ignition/gazebo/components/Imu.hh>
#include <ignition/gazebo/components/Inertial.hh>
#include <ignition/gazebo/components/Joint.hh>
#include <ignition/gazebo/components/JointAxis.hh>
#include <ignition/gazebo/components/JointForce.hh>
#include <ignition/gazebo/components/JointForceCmd.hh>
#include <ignition/gazebo/components/JointPosition.hh>
#include <ignition/gazebo/components/JointPositionReset.hh>
#include <ignition/gazebo/components/JointTransmittedWrench.hh>
#include <ignition/gazebo/components/JointType.hh>
#include <ignition/gazebo/components/JointVelocity.hh>
#include <ignition/gazebo/components/JointVelocityCmd.hh>
#include <ignition/gazebo/components/JointVelocityReset.hh>
#include <ignition/gazebo/components/LaserRetro.hh>
#include <ignition/gazebo/components/Level.hh>
#include <ignition/gazebo/components/LevelBuffer.hh>
#include <ignition/gazebo/components/LevelEntityNames.hh>
#include <ignition/gazebo/components/Lidar.hh>
#include <ignition/gazebo/components/Light.hh>
#include <ignition/gazebo/components/LightCmd.hh>
#include <ignition/gazebo/components/LightType.hh>
#include <ignition/gazebo/components/LinearAcceleration.hh>
#include <ignition/gazebo/components/LinearVelocity.hh>
#include <ignition/gazebo/components/LinearVelocityCmd.hh>
#include <ignition/gazebo/components/LinearVelocitySeed.hh>
#include <ignition/gazebo/components/Link.hh>
#include <ignition/gazebo/components/LogPlaybackStatistics.hh>
#include <ignition/gazebo/components/LogicalAudio.hh>
#include <ignition/gazebo/components/LogicalCamera.hh>
#include <ignition/gazebo/components/MagneticField.hh>
#include <ignition/gazebo/components/Magnetometer.hh>
#include <ignition/gazebo/components/Material.hh>
#include <ignition/gazebo/components/Model.hh>
#include <ignition/gazebo/components/Name.hh>
#include <ignition/gazebo/components/ParentEntity.hh>
#include <ignition/gazebo/components/ParentLinkName.hh>
#include <ignition/gazebo/components/ParticleEmitter.hh>
#include <ignition/gazebo/components/Performer.hh>
#include <ignition/gazebo/components/PerformerAffinity.hh>
#include <ignition/gazebo/components/PerformerLevels.hh>
#include <ignition/gazebo/components/Physics.hh>
#include <ignition/gazebo/components/PhysicsCmd.hh>
#include <ignition/gazebo/components/PhysicsEnginePlugin.hh>
#include <ignition/gazebo/components/Pose.hh>
#include <ignition/gazebo/components/PoseCmd.hh>
#include <ignition/gazebo/components/RenderEngineGuiPlugin.hh>
#include <ignition/gazebo/components/RenderEngineServerHeadless.hh>
#include <ignition/gazebo/components/RenderEngineServerPlugin.hh>
#include <ignition/gazebo/components/RgbdCamera.hh>
#include <ignition/gazebo/components/Scene.hh>
#include <ignition/gazebo/components/SegmentationCamera.hh>
#include <ignition/gazebo/components/SelfCollide.hh>
#include <ignition/gazebo/components/SemanticLabel.hh>
#include <ignition/gazebo/components/Sensor.hh>
#include <ignition/gazebo/components/Serialization.hh>
#include <ignition/gazebo/components/SlipComplianceCmd.hh>
#include <ignition/gazebo/components/SourceFilePath.hh>
#include <ignition/gazebo/components/SphericalCoordinates.hh>
#include <ignition/gazebo/components/Static.hh>
#include <ignition/gazebo/components/Temperature.hh>
#include <ignition/gazebo/components/TemperatureRange.hh>
#include <ignition/gazebo/components/ThermalCamera.hh>
#include <ignition/gazebo/components/ThreadPitch.hh>
#include <ignition/gazebo/components/Transparency.hh>
#include <ignition/gazebo/components/Visibility.hh>
#include <ignition/gazebo/components/Visual.hh>
#include <ignition/gazebo/components/Volume.hh>
#include <ignition/gazebo/components/Wind.hh>
#include <ignition/gazebo/components/WindMode.hh>
#include <ignition/gazebo/components/World.hh>
#include <ignition/gazebo/detail/BaseView.hh>
#include <ignition/gazebo/detail/ComponentStorageBase.hh>
#include <ignition/gazebo/detail/EntityComponentManager.hh>
#include <ignition/gazebo/detail/View.hh>
#include <ignition/gazebo/gui/Gui.hh>
#include <ignition/gazebo/gui/GuiEvents.hh>
#include <ignition/gazebo/gui/GuiSystem.hh>
#include <ignition/gazebo/rendering/Events.hh>
#include <ignition/gazebo/rendering/MarkerManager.hh>
#include <ignition/gazebo/rendering/RenderUtil.hh>
#include <ignition/gazebo/rendering/SceneManager.hh>
#include <ignition/gazebo/Conversions.hh>
#include <ignition/gazebo/Entity.hh>
#include <ignition/gazebo/EntityComponentManager.hh>
#include <ignition/gazebo/EventManager.hh>
#include <ignition/gazebo/Events.hh>
#include <ignition/gazebo/Link.hh>
#include <ignition/gazebo/Model.hh>
#include <ignition/gazebo/SdfEntityCreator.hh>
#include <ignition/gazebo/Server.hh>
#include <ignition/gazebo/ServerConfig.hh>
#include <ignition/gazebo/System.hh>
#include <ignition/gazebo/SystemLoader.hh>
#include <ignition/gazebo/SystemPluginPtr.hh>
#include <ignition/gazebo/TestFixture.hh>
#include <ignition/gazebo/Types.hh>
#include <ignition/gazebo/Util.hh>
#include <ignition/gazebo/World.hh>

